const async = require('async');

const JWT = require('jsonwebtoken');
const { JWT_SECRET, ISS_JWT } = require('../../config/secrets');
const { sendEmail } = require('../../services/mailer');
const { version } = require('../../config/public_routes');

const User = require('../user/services');
const urlGenerator = require('../../helpers/url-generator');

//Internal Function

function signToken(user) {
  return JWT.sign({
    iss: ISS_JWT,
    sub: user.id,
    iat: new Date().getTime(), //current time
    exp: new Date().setDate(new Date().getDate() + 1) //current time + 1 day ahead
  }, JWT_SECRET);
}

//Public Functions

function signUp(req, res, next) {
  const { email, password } = req.value.body;

  async.waterfall([function(callback) {
    User.selectUser({ 'local.email': email }, (user) => {
      if (user) {
        return callback(new Error('Email is already in use'))
      }
      callback(null, user)
    })
  }, function(user, callback) {
    User.saveUser({
      method: 'local',
      local: { email, password },
      confirmation_link: urlGenerator.confirmationLink()
    }, (user) => {
      callback(null, user);
    })
  }, function(user, callback) {
    //Generate Confirmation Email
    sendEmail(
      null,
      email,
      'Account Confirmation',
      'auth/account-confirmation', {
        user,
        confirmation_link: `http://${req.get('host')}/${version}/auth/verify?confirmation_link=${user.confirmation_link}`
      }
    );

    //Generate Token with JWT
    const token = signToken(user);

    //Respond with token
    res.locals.data = { token };
    next();
  }], function(error, results) {
    return res.status(403).json({ error: error.message })
  })
}

function verify(req, res, next) {
  User.selectUser({ confirmation_link: req.body.confirmation_link }, (user) => {
    User.updateUser({ confirmation_link: null }, user, (user) => {
      const token = signToken(user);
      res.locals.data = { token };
      next();
    });
  });
}

function signIn(req, res, next) {
  const token = signToken(req.user);
  res.locals.data = { token };
  next();
}

function googleOAuth(req, res, next) {
  //Generate Token
  const token = signToken(req.user);
  res.locals.data = { token };
  next();
}

function facebookOAuth(req, res, next) {
  //Generate Token
  const token = signToken(req.user);
  res.locals.data = { token };
  next();
}

module.exports = {
  signUp: signUp,
  signIn: signIn,
  googleOAuth: googleOAuth,
  facebookOAuth: facebookOAuth
};
